from ordotools.tools.parts import PENTECOST_MASSES


# FIX: delete the "orations" for the commemorations

class TemporalData:

    def __init__(self):
        self.easy_data = {
            "Circumcision": {
                "code": "Circumcision",
                "rank": [3, "d II cl"],
                "color": "white",
                "mass": {"int": "Puer natus", "glo": True, "cre": True, "pre": "et Comm de Nativitate"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "festiva",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "8_Stephen": {
                "code": "8_Stephen",
                "rank": [20, "s"],
                "color": "white",
                "mass": {"int": "Sederunt", "glo": True, "cre": False, "pre": "de Nativitate"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "8_John": {
                "code": "8_John",
                "rank": [20, "s"],
                "color": "red",
                "mass": {"int": "Introit", "glo": True, "cre": True, "pre": "Communis"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "8_Innocents": {
                "code": "8_Innocents",
                "rank": [20, "s"],
                "color": "red",
                "mass": {"int": "Ex ore infantium", "glo": True, "cre": True, "pre": "Communis"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "SNameJesus": {
                "code": "SNameJesus",
                "rank": [10, "d II cl"],
                "color": "white",
                "mass": {"int": "In nomine Jesu", "glo": True, "cre": True, "pre": "de Nativitate"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "festiva",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            # FIX: test this
            "SNameJesu_8_Ste": {
                "code": "SNameJesu_8_Ste",
                "rank": [10, "d II cl"],
                "color": "white",
                "mass": {"int": "In nomine Jesu", "glo": True, "cre": True, "pre": "de Nativitate"},
                "com_1": {
                    "code": "8_Stephen",
                    "rank": [20, "s"],
                    "color": "white",
                    "mass": {"int": "Sederunt", "glo": True, "cre": False, "pre": "de Nativitate"},
                    "matins": {},
                    "lauds": {},
                    "prime": {},
                    "little_hours": {},
                    "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                    "compline": {},
                    "office_type": False,
                    "nobility": (0, 0, 0, 0, 0, 0,),
                    "fasting": False,
                },
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "festiva",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },

            # Epiphany Season
            "V_Epiphany": {
                "code": "V_Epiphany",
                "rank": [12, "sd Vig privil 2 cl"],
                "color": "purple",
                "mass": {"int": "Dum medium silentium", "glo": True, "cre": True, "pre": "de Nativitate"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "feria",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "Epiphany": {
                "code": "Epiphany",
                "rank": [2, "d I cl cum Oct privil 2 ord"],
                "color": "white",
                "mass": {"int": "Ecce advenit", "glo": True, "cre": True, "pre": "de Epiphania"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "festiva",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "8_Epiphany": {
                "code": "8_Epiphany",
                "rank": [13, "dm"],
                "color": "white",
                "mass": {"int": "Ecce advenit", "glo": True, "cre": True, "pre": "Communis"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (3, 2, 6, 1, 3, 0), # TODO: what is the solemnity?
                "fasting": False,
            },
            "D_Epiphany": {
                "code": "D_Epiphany",
                "rank": [13, "dm"],
                "color": "white",
                "mass": {"int": "Ecce advenit", "glo": True, "cre": True, "pre": "et Comm de Epiphania"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },

            # Holy Family
            "HolyFamily": {
                "code": "HolyFamily",
                "rank": [11, "dm"],
                "color": "white",
                "mass": {"int": "Exultat", "glo": True, "cre": True, "pre": "et communcantes de Epiphania"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "D_HolyFamily": {
                "code": "D_HolyFamily",
                "rank": [11, "dm"],
                "color": "white",
                "mass": {"int": "In excelso", "glo": True, "cre": True, "pre": "et Comm de Epiphania"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },

            ##      if x == "I in Quadragesima":

            # Ash Wednesday and the follwing
            "de_AshWed": {
                "code": "de_AshWed",
                "rank": [3, "s I cl"],
                "color": "purple",
                "mass": {"int": "Misereris", "glo": True, "cre": True, "pre": "de Quadragesima"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "feria",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": True,
            },
            "AshWed_f5": {
                "code": "AshWed_f5",
                "rank": [19, "s"],
                "color": "purple",
                "mass": {"int": "Dum clamarem", "glo": False, "cre": False, "pre": "de Quadragesima"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "feria",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": True,
            },
            "AshWed_f6": {
                "code": "AshWed_f6",
                "rank": [19, "s"],
                "color": "purple",
                "mass": {"int": "Audivit", "glo": False, "cre": False, "pre": "de Quadragesima"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "feria",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": True,
            },
            "AshWed_fs": {
                "code": "AshWed_fs",
                "rank": [19, "s"],
                "color": "purple",
                "mass": {"int": "Audivit", "glo": False, "cre": False, "pre": "de Quadragesima"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "feria",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": True,
            },

            "D_Lent_1": {
                "code": "D_Lent_1",
                "rank": [1, "sd I cl"],
                "color": "purple",
                "mass": {"int": "Invocabit me", "glo": False, "cre": True, "pre": "de Quadragesima"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "dominica",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },

            # Ember days in Lent
            "Ember_Lent_4": {
                "code": "Ember_Lent_4",
                "rank": [19, "s"],
                "color": "purple",
                "mass": {"int": "Reminiscere", "glo": False, "cre": False, "pre": "de Quadragesima"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": True,
            },
            "Ember_Lent_6": {
                "code": "Ember_Lent_6",
                "rank": [19, "s"],
                "color": "purple",
                "mass": {"int": "De necessitatibus", "glo": False, "cre": False, "pre": "de Quadragesima"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": True,
            },
            "Ember_Lent_s": {
                "code": "Ember_Lent_s",
                "rank": [19, "s"],
                "color": "purple",
                "mass": {"int": "Intret", "glo": False, "cre": False, "pre": "de Quadragesima"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": True,
            },

            "SevenSorrows": {
                "code": "SevenSorrows",
                "rank": [14, "dm"],
                "color": "purple",
                "mass": {"int": "Stabant", "glo": False, "seq": "Stabat Mater", "cre": True, "pre": "de B. Maria Virg."},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": True,
            },

            # Holy Week Ferias:
            # TODO: check the verbose rank
            "de_Palm_f2": {
                "code": "de_Palm_f2",
                "rank": [3, "s"],
                "color": "purple",
                "mass": {"int": "Judica, Domine", "glo": False, "cre": False, "pre": "de Cruce"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (9, 2, 6, 13, 1, 0,),
                "fasting": True,
            },
            "de_Palm_f3": {
                "code": "de_Palm_f3",
                "rank": [3, "s"],
                "color": "purple",
                "mass": {"int": "Nos autem", "glo": False, "cre": False, "pre": "de Cruce"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (9, 2, 6, 13, 1, 0,),
                "fasting": True,
            },
            "de_Palm_f4": {
                "code": "de_Palm_f4",
                "rank": [3, "s"],
                "color": "purple",
                "mass": {"int": "In nomine Jesu", "glo": False, "cre": False, "pre": "de Cruce"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": True,
            },

            # Triduum:
            "de_Palm_f5": {
                "code": "de_Palm_f5",
                "rank": [2, "d I cl"],
                "color": "white",
                "mass": {"int": "Nos autem", "glo": True, "cre": False, "pre": "de Cruce"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": True,
            },
            "de_Palm_f6": {
                "code": "de_Palm_f6",
                "rank": [2, "d I cl"],
                "color": "black",
                "mass": {"int": "Haec dicit", "glo": False, "cre": False, "pre": ""},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": True,
            },
            "de_Palm_fs": {
                "code": "de_Palm_fs",
                "rank": [2, "d I cl"],
                "color": "purple",
                "mass": {"int": "In Missa", "glo": True, "cre": False, "pre": "Te quidem"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": True,
            },

            # Easter Week TODO: Easter Vespers:
            "Easter": {
                "code": "Easter",
                "rank": [1, "d I cl cum Oct privil I ord"],
                "color": "white",
                "mass": {"int": "Ressurexi", "glo": True, "seq": "Victimae paschali laudes", "cre": True, "pre": "Paschalis"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "festiva",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "8Easter_f2": {
                "code": "8Easter_f2",
                "rank": [2, "d I cl"],
                "color": "white",
                "mass": {"int": "Introduxit", "glo": True, "seq": "Victimae paschali laudes", "cre": True, "pre": "et Comm et Hanc Igitur, ut in die Paschae"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "8Easter_f3": {
                "code": "8Easter_f3",
                "rank": [2, "d I cl"],
                "color": "white",
                "mass": {"int": "Aqua sapientiae", "glo": True, "seq": "Victimae paschali laudes", "cre": True, "pre": "et Comm et Hanc Igitur, ut in die Paschae"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "8Easter_f4": {
                "code": "8Easter_f4",
                "rank": [3, "sd"],
                "color": "white",
                "mass": {"int": "Venite", "glo": True, "seq": "Victimae paschali laudes", "cre": True, "pre": "et Comm et Hanc Igitur, ut in die Paschae"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "8Easter_f5": {
                "code": "8Easter_f5",
                "rank": [3, "sd"],
                "color": "white",
                "mass": {"int": "Victricem", "glo": True, "seq": "Victimae paschali laudes", "cre": True, "pre": "et Comm et Hang Igitur, ut in die Paschae"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "8Easter_f6": {
                "code": "8Easter_f6",
                "rank": [3, "sd"],
                "color": "white",
                "mass": {"int": "Eduxit eos", "glo": True, "seq": "Victimae paschali laudes", "cre": True, "pre": "et Comm et Hanc Igitur, ut in die Paschae"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "WhitSaturday": {
                "code": "WhitSaturday",
                "rank": [3, "sd"],
                "color": "white",
                "mass": {"int": "Eduxit Dominus", "glo": True, "seq": "Victimae paschali laudes", "cre": True, "pre": "et Comm et Hanc Igitur, ut in die Paschae"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },

            # Solemnity of St. Joseph  TODO: add the octave
            "StJoseph": {
                "code": "StJoseph", #, C. et Ecclesiæ Universalis Patroni",
                "rank": [2, "d I cl cum Oct Communi"],
                "color": "white",
                "mass": {"int": "Justus ut palma", "glo": True, "cre": True, "pre": "de S Joseph"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (1, 2, 4, 4, 1, 0),
                "fasting": False,
            },
            "8_StJoseph": {
                "code": "8_StJoseph",
                "rank": [13, "dm"],
                "color": "white",
                "mass": {"int": "Missa", "glo": True, "cre": True, "pre": "Communis"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },

            # Rogation Days
            "Rogation_1": {
                "code": "Rogation_1",
                "rank": [19, "feria"],
                "color": "purple",
                "mass": {"int": "Exaudivit", "glo": False, "cre": True, "pre": "Paschalis"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "feria",
                "nobility": (0, 0, 0, 13, 0, 0,),
                "fasting": False,
            },
            "Rogation_2": {
                "code": "Rogation_2",
                "rank": [23, "feria"],
                "color": "purple",
                "mass": {"int": "Exaudivit", "glo": False, "cre": True, "pre": "Paschalis"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "feria",
                "nobility": (0, 0, 0, 13, 0, 0,),
                "fasting": False,
            },
            "Rogation_3": {
                "code": "Rogation_3",
                "rank": [23, "feria"],
                "color": "purple",
                "mass": {"int": "Exaudivit", "glo": False, "cre": True, "pre": "Paschalis"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "feria",
                "nobility": (9, 2, 6, 13, 3, 0,),
                "fasting": False,
            },


            # Ascension
            "Ascension": {
                "code": "Ascension",
                "rank": [2, "d I cl cum Oct privil 3 ord"],
                "color": "white",
                "mass": {"int": "Viri Galilæi", "glo": True, "cre": True, "pre": "et Comm de Ascensione"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "festiva",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "8_Ascension": {
                "code": "8_Ascension",
                "rank": [13, "dm"],
                "color": "white",
                "mass": {"int": "Viri Galilæi", "glo": True, "cre": True, "pre": "et Comm de Ascensione"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (3, 2, 3, 1, 3, 0),
                "fasting": False,
            },

            # TODO: this day has special rules
            "p_Ascension_f6": {
                "code": "p_Ascension_f6",
                "rank": [13, "dm"],
                "color": "white",
                "mass": {"int": "Viri Galilæi", "glo": True, "cre": True, "pre": "et Comm de Ascensione"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },

            # TODO: see if this is right
            "D_Ascension": {
                "code": "D_Ascension",
                "rank": [16, "sd"], # FIX: check the rank and the Introit
                "color": "white",
                "mass": {"int": "Exaudi, Domine", "glo": True, "cre": False, "pre": "de Ascensione"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (5, 1, 6, 13, 3, 0,),
                "fasting": False,
            },

            "S_8_Ascension": {
                "code": "S_8_Ascension",
                "rank": [17, "sd"],
                "color": "white",
                "mass": {"int": "Exaudi, Domine", "glo": True, "cre": False, "pre": "de Ascensione"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },

            "WhitSunday": {
                "code": "WhitSunday",
                "rank": [1, "dm I cl"],
                "color": "white",
                "mass": {"int": "Quasi modo", "glo": True, "cre": True, "pre": "Paschalis"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "dominica",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,

            },

            "V_Pentecost": {
                "code": "V_Pentecost",
                "rank": [3, "d I cl Vig privil I cl"],
                "color": "red",
                "mass": {"int": "Cum sanctificatus", "glo": True, "cre": False, "pre": "et Comm et Hanc Igitur de Pentecoste"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": True,
            },

            # Pentecost Week
            "Pentecost": {
                "code": "Pentecost",
                "rank": [1, "d I cl cum Oct privil I ord"],
                "color": "red",
                "mass": {"int": "Spiritus Domini", "glo": True, "seq": "Veni, Sancte Spiritus", "cre": True, "pre": "et Comm et Hanc Igitur de Pentecoste"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "festiva",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            # TODO: see if the first two ferias are 2 or 3
            "8Pent_f2": {
                "code": "8Pent_f2",
                "rank": [3, "d I cl"],
                "color": "red",
                "mass": {"int": "Cibavit eos", "glo": True, "cre": True, "seq": "Veni, Sancte Spiritus", "pre": "et Comm et Hanc Igitur de Pentecoste"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "feria",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "8Pent_f3": {
                "code": "8Pent_f3",
                "rank": [3, "d I cl"],
                "color": "red",
                "mass": {"int": "Cibavit eos", "glo": True, "cre": True, "seq": "Veni, Sancte Spiritus", "pre": "et Comm et Hanc Igitur de Pentecoste"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "feria",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "8Pent_f5": {
                "code": "8Pent_f5",
                "rank": [3, "d I cl"],
                "color": "red",
                "mass": {"int": "Accepite jucunditatem", "glo": True, "seq": "Veni, Sancte Spiritus", "cre": True, "pre": "et Comm et Hanc Igitur de Pentecoste"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "feria",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },

            "Ember_Pent_4": {
                "code": "Ember_Pent_4",
                "rank": [3, "sd"],
                "color": "red",
                "mass": {"int": "Deus, dum egredereris", "glo": True, "seq": "Veni, Sancte Spiritus", "cre": True, "pre": "et Comm et Hanc Igitur de Pentecoste"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "feria",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": True,
            },
            "Ember_Pent_6": {
                "code": "Ember_Pent_6",
                "rank": [3, "sd"],
                "color": "red",
                "mass": {"int": "Repleatur os meum", "glo": True, "seq": "Veni, Sancte Spiritus", "cre": True, "pre": "et Comm et Hanc Igitur de Pentecoste"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "feria",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": True,
            },
            "Ember_Pent_s": {
                "code": "Ember_Pent_s",
                "rank": [3, "sd"],
                "color": "red",
                "mass": {"int": "Caritas Dei", "glo": True, "seq": "Veni, Sancte Spiritus", "cre": True, "pre": "et Comm et Hanc Igitur de Pentecoste"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "feria",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": True,
            },

            # First three post-Pentecost Sundays
            "Trinity": {
                "code": "Trinity",
                "rank": [2, "d I cl"],
                "color": "white",
                "mass": {"int": "Benedicta sit", "glo": True, "cre": True, "pre": "de Ssma Trinitate"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "festiva",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "D_Pent_2": {
                "code": "D_Pent_2", #Christi (Dominica II post Pentecosten)",
                "rank": [9, "sd"],
                "color": "green",
                "mass": {"int": "Factus est", "glo": True, "seq": "Lauda, Sion, Salvatorem", "cre": True, "pre": "de Nativitate, vel de Ssma Trinitate"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "dominica",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "D_Pent_3": {
                "code": "D_Pent_3",  # (Dominica III post Pentecosten)",
                "rank": [12, "sd"],
                "color": "green",
                "mass": {"int": "Respice in me", "glo": True, "cre": True, "pre": "de sacratissimo Code Jesu"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "dominica",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },

            "CorpusChristi": {
                "code": "CorpusChristi",
                "rank": [2, "d I cl cum Oct privil 2 ord"],
                "color": "white",
                "mass": {"int": "Cibavit eos", "glo": True, "seq": "Lauda, Sion", "cre": True, "pre": "de Nativitate"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "festiva",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },

            "8_CorpusChristi": {
                "code": "8_CorpusChristi",
                "rank": [4, "dm"],
                "color": "white",
                "mass": {"int": "Cibavit eos", "glo": True, "seq": "Lauda, Sion", "cre": True, "pre": "de Nativitate"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },

            "SacredHeart": {
                "code": "SacredHeart",
                "rank": [2, "d I cl cum Oct privil 3 ord"],
                "color": "white",
                "mass": {"int": "Cogitationes", "glo": True, "cre": True, "pre": "de Ssmo Corde Iesu"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "festiva",
                "nobility": (1, 0, 3, 1, 1, 0),
                "fasting": False,
            },

            "8_SacredHeart": {
                "code": "8_SacredHeart",
                "rank": [13, "dm"],
                "color": "white",
                "mass": {"int": "Cogitationes", "glo": True, "cre": True, "pre": "de Ssmo Corde Iesu"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },

            "Ember_Sept_4": {
                "code": "Ember_Sept_4",
                "rank": [19, "s"],
                "color": "purple",
                "mass": {"int": "Exsultate Deo", "glo": False, "cre": False, "pre": "Communis"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "feria",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": True,
            },

            "Ember_Sept_6": {
                "code": "Ember_Sept_6",
                "rank": [19, "s"],
                "color": "purple",
                "mass": {"int": "Laetetur cor", "glo": False, "cre": False, "pre": "Communis"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "feria",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": True,
            },
            "Ember_Sept_s": {
                "code": "Ember_Sept_s",
                "rank": [19, "s"],
                "color": "purple",
                "mass": {"int": "Venite, adoremus", "glo": False, "cre": False, "pre": "Communis"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "feria",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": True,
            },

            # Christ the King
            "ChristKing": {
                "code": "ChristKing",
                "rank": [2, "d I cl"],
                "color": "white",
                "mass": {"int": "Dignus est", "glo": True, "cre": True, "pre": "de DNJC Rege"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "festiva",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },

            # Ember Days of Advent
            "Ember_Advent_4": {
                "code": "Ember_Advent_4",
                "rank": [19, "feria"], # TODO: check the rank for this
                "color": "purple",
                "mass": {"int": "Rorate cæli", "glo": False, "cre": False, "pre": "Communis"},
                "com_1": {"oration": "Deus qui de beate"},
                "com_2": {"oration": "Ecclesiæ"},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "feria",
                "nobility": (9, 2, 6, 13, 3, 0),
                "fasting": True,
            },
            "Ember_Advent_6": {
                "code": "Ember_Advent_6",
                "rank": [19, "feria"], # TODO: check the rank for this
                "color": "purple",
                "mass": {"int": "Prope es tu", "glo": False, "cre": False, "pre": "Communis"},
                "com_1": {"oration": "Deus qui de beate"},
                "com_2": {"oration": "Ecclesiæ"},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "feria",
                "nobility": (9, 2, 6, 13, 3, 0),
                "fasting": True,
            },
            "Ember_Advent_s": {
                "code": "Ember_Advent_s",
                "rank": [19, "feria"],  # TODO: check the rank for this
                "color": "purple",
                "mass": {"int": "Veni, et ostende", "glo": False, "cre": False, "pre": "Communis"},
                "com_1": {"oration": "Deus qui de beate"},
                "com_2": {"oration": "Ecclesiæ"},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "feria",
                "nobility": (9, 2, 6, 13, 3, 0),
                "fasting": True,
            },


            # Christmastide
            "V_Christmas": {
                "code": "V_Christmas",
                "rank": [3, "d I cl Vig privil I cl"],
                "color": "purple",
                "mass": {"int": "Hodie scietis", "glo": False, "cre": False, "pre": "Communis"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "feria",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": True,
            },
            "DV_Christmas": {
                "code": "DV_Christmas",
                "rank": [3, "d I cl Vig privil I cl"],
                "color": "purple",
                "mass": {"int": "Hodie scietis", "glo": False, "cre": True, "pre": "de Trinitate"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "feria",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "Christmas": {
                "code": "Christmas",
                "rank": [2, "d I cl cum Oct privil 3 ord"],
                "color": "white",
                "mass": {
                    "Ad Primam Missam": {"int": "Domine dixit", "glo": True, "cre": True, "pre": r'et Comm (in hac Missa tantum dicitur "noctem") de Nativitate'},
                    "Ad Secundam Missam": {"int": "Lux fulgebit", "glo": True, "cre": True, "pre": "et Comm de Nativitate"},
                    "Ad Tertiam Missam": {"int": "Puer natus", "glo": True, "cre": True, "pre": "et Comm de Nativitate", "proper_last_gospel": "Epiph"},
                },
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {"psalms": "sunday"},
                "prime": {"v_r": "Qui natus es"},
                "little_hours": {"psalms": "sunday"},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {"sunday": True,},
                "office_type": "festiva",
                "nobility": (1, 1, 3, 1, 1, 0),
                "fasting": False,
            },
            "D_Christmas_r": {
                "code": "D_Christmas_r",
                "rank": [12, "sd"],
                "color": "white",
                "mass": {"int": "Dum medium silentium", "glo": True, "cre": True, "pre": "de Comm de Nativitate"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "dominica",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "D_Christmas": {
                "code": "D_Christmas",
                "rank": [12, "sd"],
                "color": "white",
                "mass": {"int": "Dum medium", "glo": True, "cre": True, "pre": "et Comm de Nativitate"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "dominica",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "8_Chritmas_f6": {
                "code": "8_Chritmas_f6",
                "rank": [16, "sd"],
                "color": "white",
                "mass": {"int": "Puer natus est nobis", "glo": True, "cre": True, "pre": "et Comm de Nativitate"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""}, "office_type": "feria",
                "compline": {},
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "StStephan": {
                "code": "StStephan",
                "rank": [10, "d II cl cum Oct simplici"],
                "color": "red",
                "mass": {"int": "Sederunt", "glo": True, "cre": True, "pre": "et Comm de Nativitate"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "festiva",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "StJohn": {
                "code": "StJohn",
                "rank": [10, "d II cl cum Oct simplici"],
                "color": "white",
                "mass": {"int": "In medio ecclesiæ", "glo": True, "cre": True, "pre": "et Comm de Nativitate"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "festiva",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "StsInnocents": {
                "code": "StsInnocents",
                "rank": [10, "d II cl cum Oct simplici"],
                "color": "red",
                "mass": {"int": "Ex ore infantium", "glo": True, "cre": True, "pre": "et Comm de Nativitate"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "festiva",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "StThomas": {
                "code": "StThomas",
                "rank": [15, "d"],
                "color": "red",
                "mass": {"int": "Gaudeamus omnes", "glo": True, "cre": True, "pre": "et Comm de Nativitate"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "festiva",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "StSylvester": {
                "code": "StSylvester",
                "rank": [15, "d"],
                "color": "white",
                "mass": {"int": "Si diligis me", "glo": False, "cre": True, "pre": "et Comm de Nativitate"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "festiva",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },

            "D_StThomas": {
                "code": "D_StThomas",
                "rank": [15, "d"],
                "color": "red",
                "mass": {"int": "Gaudeamus omnes", "glo": True, "cre": True, "pre": "et Comm de Nativitate"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "festiva",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            "D_StSylvester": {
                "code": "D_StSylvester",
                "rank": [15, "d"],
                "color": "white",
                "mass": {"int": "Si diligis me", "glo": False, "cre": True, "pre": "et Comm de Nativitate"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "festiva",
                "nobility": (0, 0, 0, 0, 0, 0,),
                "fasting": False,
            },
            # TODO: add the Sundays if the fall on St. Thomas or St. Sylvester
        }

        self.data = self.easy_data |\
            self.epiphany_time() |\
            self.epiphany_octave() |\
            self.septuagesima_time() |\
            self.lent_sundays() |\
            self.paschaltime() |\
            self.ascension_ferias() |\
            self.corpus_ferias() |\
            self.sacredheart_ferias() |\
            self.solemnity_st_joseph() |\
            self.pentecost_sundays() |\
            self.pentecost_epiphany_sundays() |\
            self.last_pentecost() |\
            self.advents() |\
            self.three_weeks_after_pentecost()

    def lent_sundays(self) -> dict:
        the_days = {}
        normal_lents = [f"D_Lent_{l+1}" for l in range(4)]
        later_lents = ["D_Passion_5", "D_Palm_6"]
        lents = [*normal_lents, *later_lents]
        rank = 8
        for x, date in enumerate(lents):
            if x in [0, 1, 3]:
                rank = 1
            else:
                rank = 8
            for feria in range(7):
                if feria == 0:
                    the_days |= {
                        date: {
                            "code": date,
                            "rank": [rank, "sd I cl"],  # FIX: check this
                            "color": f"{'purple' if x+1 != 4 else 'pink'}",
                            "mass": {
                                "int": "",  # TODO: add all of the Introits
                                "glo": False,
                                "cre": False,
                                "pre": "de Quadragesima" if x < 6 else "de Cruce",
                            },
                            "com_1": {},
                            "com_2": {},
                            "com_3": {},
                            "matins": {},
                            "lauds": {},
                            "prime": {},
                            "little_hours": {},
                            "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                            "compline": {},
                            "office_type": False,
                            "nobility": (0, 0, 0, 0, 0, 0,),
                            "fasting": False,
                        }
                    }
                else:
                    the_days |= {
                        f"de_{'Lent' if x < 4 else 'Passion'}_f{feria+1 if feria != 6 else 's'}": {
                            # "code": f"Feria {integer_to_roman(feria+1)} infra Hebd {integer_to_roman(x+1)} in Quadragesima",
                            "code": f"de_{'Lent' if x < 4 else 'Passion'}_f{feria+1 if feria != 6 else 's'}",
                            "rank": [19, "feria"],
                            "color": "purple",
                            "mass": {
                                "int": "", # TODO: add all of the Lent feria Introits
                                "glo": False,
                                "cre": False,
                                "pre": "de Quadragesima" if x < 6 else "de Cruce",
                            },
                            "com_1": {},
                            "com_2": {},
                            "com_3": {},
                            "matins": {},
                            "lauds": {},
                            "prime": {},
                            "little_hours": {},
                            "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                            "compline": {},
                            "office_type": False,
                            "nobility": (9, 2, 6, 13, 3, 0), # FIX: Lent has higher ranking ferias?
                            "fasting": True,
                        }
                    }
        return the_days

    def ascension_ferias(self) -> dict:
        return { # NOTE: there are duplicates, but does it matter?
            f"in_8_Ascension_{date}": {
                "code": f"in_8_Ascension_{date}",
                "rank": [16, "sd"],
                "color": "white",
                "mass": {"int": "Viri galilæi", "glo": True, "cre": False, "pre": "de Ascensione"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": False,
                "nobility": (9, 2, 6, 13, 3, 0),
                "fasting": False,
            } for date in range(1,8)
        }

    def corpus_ferias(self) -> dict:
        return {
            f"{date+1}_in_8_CorpusChristi": {
                "code": f"{date+1}_in_8_CorpusChristi",
                "rank": [9, "sd"],
                "color": "white",
                "mass": {"int": "Cibavit eos", "glo": True, "seq": "Lauda, Sion", "cre": True, "pre": "de Nativitate"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "festiva",
                "nobility": (9, 2, 6, 13, 3, 0),
                "fasting": False,
            } for date in range(1,8)
        }

    def sacredheart_ferias(self) -> dict:
        return {
            f"{date+1}_in_8_SacredHeart": {
                "code": f"{date+1}_in_8_SacredHeart",
                "rank": [18, "sd"], # TODO: verify this rank
                "color": "white",
                "mass": {"int": "Cogitationes", "glo": True, "cre": True, "pre": "de Ssmo Corde Iesu"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "festiva",
                "nobility": (1, 0, 3, 1, 1, 0),
                "fasting": False,
            } for date in range(1,8)
        }


    def pentecost_sundays(self) -> dict:
        # TODO: add the first 4 Sundays after Pentecost (excluding Trinity)
        pentecost_season = {}
        for date in range(4, 29):
            pentecost_season |= {
                f"D_Pent_{date}": {
                    "code": f"D_Pent_{date}",
                    "rank": [12, "sd"],
                    "color": "green",
                    "mass": {"int": f"{PENTECOST_MASSES[date-5] if date >= 4 else ''}", "glo": True, "cre": True, "pre": "de Trinitate"},
                    "com_1": {"oration": "A cunctis"},
                    "com_2": {"oration": "ad libitum"},
                    "com_3": {},
                    "matins": {},
                    "lauds": {},
                    "prime": {},
                    "little_hours": {},
                    "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                    "compline": {},
                    "office_type": "dominica",
                    "nobility": (0, 0, 0, 0, 0, 0,),
                    "fasting": False,
                }
            }
            for feria in range(6):
                pentecost_season |= {
                    f"de_Pent_{date}_f{feria+2 if feria != 5 else 's'}": {
                        "code": f"de_Pent_{date}_f{feria+2 if feria != 5 else 's'}",
                        "rank": [23, "sd"],
                        "color": "green",
                        "mass": {"int": f"{PENTECOST_MASSES[date-5] if date >= 4 else ''}", "glo": True, "cre": False, "pre": "Communi"},
                        "com_1": {"oration": "A cunctis"},
                        "com_2": {"oration": "ad libitum"},
                        "com_3": {},
                        "matins": {},
                        "lauds": {},
                        "prime": {},
                        "little_hours": {},
                        "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                        "compline": {},
                        "office_type": "dominica",
                        "nobility": (9, 2, 6, 13, 3, 0),
                        "fasting": False,
                    }
                }
        return pentecost_season

    def pentecost_epiphany_sundays(self) -> dict:
        """
        This is really rough and might not be too efficient, but
        it works for now.
        """
        epiphany_pents = {}
        for pent in range(22, 29):
            for epiph in range(3, 7):
                epiphany_pents |= {
                    f"D_Epiph_{epiph}_{pent}": {
                        "code": f"D_Epiph_{epiph}_{pent}",
                        "rank": [12, "sd"],
                        "color": "green",
                        "mass": {"int": "Dicit Dominus", "glo": True, "cre": True, "pre": "de Trinitate"},
                        "com_1": {"oration": "A cunctis"},
                        "com_2": {"oration": "ad libitum"},
                        "com_3": {},
                        "matins": {},
                        "lauds": {},
                        "prime": {},
                        "little_hours": {},
                        "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                        "compline": {},
                        "office_type": "dominica",
                        "nobility": (0, 0, 0, 0, 0, 0,),
                        "fasting": False,
                    }
                }
                for feria in range(6):
                    epiphany_pents |= {
                        f"de_Epiph_{epiph}_{pent}_f{feria+2 if feria != 5 else 's'}": {
                            "code": f"de_Epiph_{epiph}_{pent}_f{feria+2 if feria != 5 else 's'}",
                            "rank": [23, "feria"],
                            "color": "green",
                            "mass": {"int": "Dicit Dominus", "glo": True, "cre": False, "pre": "Communi"},
                            "com_1": {"oration": "A cunctis"},
                            "com_2": {"oration": "ad libitum"},
                            "com_3": {},
                            "matins": {},
                            "lauds": {},
                            "prime": {},
                            "little_hours": {},
                            "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                            "compline": {},
                            "office_type": "dominica",
                            "nobility": (9, 2, 6, 13, 3, 0),
                            "fasting": False,
                        }
                    }
        return epiphany_pents

    def last_pentecost(self) -> dict:
        last_pents = {}
        for pent in range(23, 29):
            last_pents |= {
                f"D_UltPent_{pent}": {
                    "code": f"D_UltPent_{pent}",
                    "rank": [12, "sd"],
                    "color": "green",
                    "mass": {"int": PENTECOST_MASSES[-1], "glo": True, "cre": True, "pre": "de Trinitate"},
                    "com_1": {},
                    "com_2": {},
                    "com_3": {},
                    "matins": {},
                    "lauds": {},
                    "prime": {},
                    "little_hours": {},
                    "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                    "compline": {},
                    "office_type": "dominica",
                    "nobility": (0, 0, 0, 0, 0, 0,),
                    "fasting": False,
                }
            }
            for feria in range(6):
                last_pents |= {
                    f"de_UltPent_{pent}_f{feria+2 if feria != 5 else 's'}": {
                        "code": f"de_UltPent_{pent}_f{feria+2 if feria != 5 else 's'}",
                        "rank": [23, "feria"],
                        "color": "green",
                        "mass": {"int": PENTECOST_MASSES[-1], "glo": True, "cre": False, "pre": "Communi"},
                        "com_1": {},
                        "com_2": {},
                        "com_3": {},
                        "matins": {},
                        "lauds": {},
                        "prime": {},
                        "little_hours": {},
                        "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                        "compline": {},
                        "office_type": "dominica",
                        "nobility": (9, 2, 6, 13, 3, 0),
                        "fasting": False,
                    }
                }
        return last_pents

    def advents(self) -> dict:
        advent_season = {}
        advent_sundays = [
            "Ad te levavi",
            "Populus Sion",
            "Gaudete",
            "Rorate cæli",
        ]
        for x, introit in enumerate(advent_sundays):
            advent_season |= {
                f"D_Advent_{x+1}": { # TODO: verify that 2-4 advents are minor sundays
                                    "code": f"D_Advent_{x+1}",
                                    "rank": [1 if x == 0 else 12, f"{'sd I cl' if x == 0 else 'sd II cl'}"],
                                    "color": f"{'purple' if x+1 != 3 else 'pink'}",
                                    "mass": {"int": f"{introit}", "glo": False, "cre": True, "pre": "de Trinitate"},
                                    "com_1": {"oration": "Deus qui de beate"},
                                    "com_2": {"oration": "Ecclesiæ"},
                                    "com_3": {},
                                    "matins": {},
                                    "lauds": {},
                                    "prime": {},
                                    "little_hours": {},
                                    "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                                    "compline": {},
                                    "office_type": "dominica",
                                    "nobility": (0, 0, 0, 0, 0, 0,),
                                    "fasting": False,
                                    }
            }
            for feria in range(6):
                advent_season |= {
                    f"de_Advent_{x+1}_f{feria+2 if feria != 5 else 's'}": {
                        "code": f"de_Advent_{x+1}_f{feria+2 if feria != 5 else 's'}",
                        "rank": [23, "feria"], # FIX: change the rank
                        "color": "purple",
                        "mass": {"int": f"{introit}", "glo": False, "cre": True, "pre": "de Trinitate"},
                        "com_1": {"oration": "Deus qui de beate"},
                        "com_2": {"oration": "Ecclesiæ"},
                        "com_3": {},
                        "matins": {},
                        "lauds": {},
                        "prime": {},
                        "little_hours": {},
                        "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                        "compline": {},
                        "office_type": "feria",
                        "nobility": (9, 2, 6, 13, 3, 0),
                        "fasting": False,
                    }
                }
        return advent_season

    def three_weeks_after_pentecost(self) -> dict:
        # NOTE: this is overkill, but it works, and is probably faster than a fix
        weeks = {}
        introits = [
            "Benedicta sit",
            "Factus est",
            "Respice in me",
        ]
        for x, introit in enumerate(introits):
            for feria in range(6):
                weeks |= {
                    f"de_Pent_{x+1}_f{feria+2 if feria != 5 else 's'}": {
                        "code": f"de_Pent_{x+1}_f{feria+2 if feria != 5 else 's'}",
                        "rank": [23, "feria"], # FIX: change the ranking?
                        "color": "green", # TODO: check if this is right for Trinity
                        "mass": {"int": introit, "glo": True, "seq": "Lauda, Sion, Salvatorem", "cre": False, "pre": "de Trinitate"},
                        "com_1": {},
                        "com_2": {},
                        "com_3": {},
                        "matins": {},
                        "lauds": {},
                        "prime": {},
                        "little_hours": {},
                        "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                        "compline": {},
                        "office_type": "ferial",
                        "nobility": (9, 2, 6, 13, 3, 0),
                        "fasting": False,
                    },
                }
        return weeks

    def paschaltime(self) -> dict:
        paschaltime = {}
        for week in range(6):
            paschaltime |= {
                f"D_Easter_{week+1}": {
                    "code": f"D_Easter_{week+1}",
                    "rank": [12, "sd"],
                    "color": "white",
                    "mass": {"int": "Missa", "glo": True, "cre": True, "pre": "Paschalis"},
                    "com_1": {},
                    "com_2": {},
                    "com_3": {},
                    "matins": {},
                    "lauds": {},
                    "prime": {},
                    "little_hours": {},
                    "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                    "compline": {},
                    "office_type": "dominica",
                    "nobility": (0, 0, 0, 0, 0, 0,),
                    "fasting": False,
                }
            }
            for feria in range(6):
                pass
                paschaltime |= {
                    f"de_Easter_{week+1}_f{feria+2 if feria != 5 else 's'}": {
                        "code": f"de_Easter_{week+1}_f{feria+2 if feria != 5 else 's'}",
                        "rank": [23, "feria"], # FIX: check the ranking
                        "color": "white",
                        "mass": {"int": "Missa", "glo": True, "cre": False, "pre": "Communis"},
                        "com_1": {},
                        "com_2": {},
                        "com_3": {},
                        "matins": {},
                        "lauds": {},
                        "prime": {},
                        "little_hours": {},
                        "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                        "compline": {},
                        "office_type": "feria",
                        "nobility": (9, 2, 6, 13, 3, 0),
                        "fasting": False,
                    }
                }
        return paschaltime

    def solemnity_st_joseph(self) -> dict:
        solemnity_ferias = {}
        for feria in range(6):
            solemnity_ferias |= {
                f"{feria+2}_in_8_StJoseph": {
                    "code": f"{feria+2}_in_8_StJoseph",
                    "rank": [18, "feria"], # FIX: check the verbose rank
                    "color": "white",
                    "mass": {"int": "Justus ut palma", "glo": True, "cre": True, "pre": "de S Joseph"},
                    "com_1": {},
                    "com_2": {},
                    "com_3": {},
                    "matins": {},
                    "lauds": {},
                    "prime": {},
                    "little_hours": {},
                    "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                    "compline": {},
                    "office_type": False,
                    "nobility": (9, 2, 6, 13, 3, 0),
                    "fasting": False,
                }
            }
        return solemnity_ferias

    def septuagesima_time(self) -> dict:
        septuagesima = {}
        for x, sunday in enumerate(["Septuagesima", "Sexagesima", "Quinquagesima"]):
            septuagesima |= {
                sunday: {
                    "code": sunday,
                    "rank": [8, "sd II cl"],
                    "color": "purple",
                    "mass": {"int": "", "glo": False, "cre": True, "pre": "Communis"},
                    "com_1": {},
                    "com_2": {},
                    "com_3": {},
                    "matins": {},
                    "lauds": {},
                    "prime": {},
                    "little_hours": {},
                    "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                    "compline": {},
                    "office_type": "dominica",
                    "nobility": (0, 0, 0, 0, 0, 0,),
                    "fasting": False,
                }
            }
            for feria in range(6):
                septuagesima |= {
                    f"de_{sunday[:4]}_f{feria+2 if feria != 5 else 's'}": {
                        "code": f"de_{sunday[:4]}_f{feria+2 if feria != 5 else 's'}",
                        "rank": [23, "feria"],
                        "color": "purple",
                        "mass": {"int": "", "glo": False, "cre": True, "pre": "Communis"},
                        "com_1": {},
                        "com_2": {},
                        "com_3": {},
                        "matins": {},
                        "lauds": {},
                        "prime": {},
                        "little_hours": {},
                        "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                        "compline": {},
                        "office_type": "dominica",
                        "nobility": (9, 2, 6, 13, 3, 0),
                        "fasting": False,
                    }
                }
        return septuagesima

    def epiphany_time(self) -> dict:
        epiphany = {}
        for sunday in range(6):
            epiphany |= {
                f"D_Epiph_{sunday+1}": {
                    "code": f"D_Epiph_{sunday+1}",
                    "rank": [12, "sd"],
                    "color": "green",
                    "mass": {"int": "Omnis terra", "glo": True, "cre": True, "pre": "de Ssma Trinitate"},
                    "com_1": {},
                    "com_2": {},
                    "com_3": {},
                    "matins": {},
                    "lauds": {},
                    "prime": {},
                    "little_hours": {},
                    "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                    "compline": {},
                    "office_type": "dominica",
                    "nobility": (0, 0, 0, 0, 0, 0,),
                    "fasting": False,
                }
            }
            for feria in range(6):
                epiphany |= {
                    f"de_Epiph_{sunday+1}_{feria+2 if feria != 5 else 's'}": {
                        "code": f"de_Epiph_{sunday+1}_{feria+2 if feria != 5 else 's'}",
                        "rank": [23, "feria"],
                        "color": "white",
                        "mass": {"int": "Omnis terra", "glo": True, "cre": False, "pre": "Communis"},
                        "com_1": {},
                        "com_2": {},
                        "com_3": {},
                        "matins": {},
                        "lauds": {},
                        "prime": {},
                        "little_hours": {},
                        "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                        "compline": {},
                        "office_type": "feria",
                        "nobility": (8, 2, 6, 13, 3, 0,),
                        "fasting": False,
                    }
                }
        return epiphany

    def epiphany_octave(self) -> dict:
        octave = {
            "8_Epiph_fs": {
                "code": "8_Epiph_fs",
                "rank": [9, "feria"],
                "color": "white",
                "mass": {"int": "Ecce advenit", "glo": True, "cre": True, "pre": "Communis"},
                "com_1": {},
                "com_2": {},
                "com_3": {},
                "matins": {},
                "lauds": {},
                "prime": {},
                "little_hours": {},
                "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                "compline": {},
                "office_type": "feria",
                "nobility": (8, 2, 6, 13, 3, 0,), # FIX: check this rank
                "fasting": False,
            },
        }
        for feria in range(6):
            octave |= {
                f"8_Epiph_f{feria+2}": {
                    "code": f"8_Epiph_f{feria+2}",
                    "rank": [9, "feria"],
                    "color": "white",
                    "mass": {"int": "Ecce advenit", "glo": True, "cre": True, "pre": "Communis"},
                    "com_1": {},
                    "com_2": {},
                    "com_3": {},
                    "matins": {},
                    "lauds": {},
                    "prime": {},
                    "little_hours": {},
                    "vespers": {"proper": False, "admag": ["firstVespers", "secondVespers"], "propers": {}, "oration": ""},
                    "compline": {},
                    "office_type": "feria",
                    "nobility": (8, 2, 6, 13, 3, 0,), # FIX: check this rank
                    "fasting": False,
                }
            }
        return octave



from datetime import datetime
from datetime import timedelta

import dateutil.easter

import re

import functools


@functools.lru_cache()
class LiturgicalYearMarks:

    def __init__(self, year):
        self.year = year
        self.christmas = datetime.strptime(str(self.year) + "-12-25", "%Y-%m-%d")
        self.first_advent = self.christmas - findsunday(self.christmas) - timedelta(weeks=3)
        self.last_advent = self.christmas - timedelta(days=1)

        # FIX: there is somethig wrong here, right? Not used anywhere...
        self.easter_season_start = easter(self.year) - timedelta(weeks=6, days=4)
        
        self.lent_begins = easter(self.year) - timedelta(weeks=6, days=4)
        self.lent_ends = easter(self.year) - timedelta(days=1)
        self.easter = easter(self.year)
        self.easter_season_end = easter(self.year) + timedelta(days=39)
        self.pentecost_season_start = easter(self.year) + timedelta(days=49)
        self.pentecost_season_end = self.first_advent - timedelta(days=1)


def findsunday(date: datetime) -> timedelta:
    """
    return the distance betweent the date and
    the previous Sunday, as timedelta.days
    """
    return timedelta(days=int(date.strftime('%w')))


@functools.lru_cache()
def easter(year: int) -> datetime:
    """ return the date of easter for this year """
    return datetime(
        year=int(dateutil.easter.easter(year).strftime('%Y')),
        month=int(dateutil.easter.easter(year).strftime('%m')),
        day=int(dateutil.easter.easter(year).strftime('%d'))
    )


ladys_office = {
    "code": "lady_saturday",
    "rank": [21, "s"],
    "color": "white",
    "mass": {
        "int": "Salve sancta parens",
        "glo": True,
        "cre": False,
        "pre": "de B Maria Virg (et te in Veneratione)"
    },
    "com_1": {"oration": "Deus qui corda"},
    "com_2": {"oration": "Ecclesiæ"},
    "com_3": {},
    "matins": {},
    "lauds": {},
    "prime": {"responsory": "Qui natus est", "preces": True},
    "little_hours": {},
    "vespers": {
        "proper": False,
        "admag": ("firstVespers", "secondVespers"),
        "propers": {},
        "oration": ""
    },
    "compline": {},
    "office_type": "ut in pr loco",
    "nobility": (8, 2, 6, 13, 3, 0,),
    "fasting": False,
}


def day(year: int, month: int, day: int) -> datetime:
    return datetime(year=year, month=month, day=day)


def weeks(i: int) -> timedelta:
    """ return a timedelta week, with integers as the input """
    return timedelta(weeks=i)


def days(numdays: int) -> timedelta:
    """ return a timedelta day(s), with integers as the input """
    return timedelta(days=numdays)


def weekday(date: datetime) -> str:
    """ return the weekday, with datetime as the input """
    return date.strftime('%a')


# def advance_a_day(date: str) -> str:
#     """ return a date advanced a day, returns a string mm/dd """
#     return date + timedelta(days=1)


# def find_extra_epiphany(pents: int) -> int:
#     """ return the number of Sundays not celebrated after Epiphany """
#     if pents == 23:
#         pass
#     else:
#         return pents - 24


def leap_year(year: int) -> bool:
    """ return true if year is a leap year """
    if (year % 4) == 0:
        if (year % 100) == 0:
            if (year % 400) == 0:
                return True
            else:
                return False
        else:
            return True
    else:
        return False


def latex_replacement(string: str) -> str:
    """ return a string formatted for LaTeX with escape characters """
    return re.sub('&', r'\&', re.sub('_', r'\_', string))


def translate_month(month: str) -> str:
    """ return the latin name for a given english month """
    months_in_latin = {
        'January': 'Januarius',
        'February': 'Februarius',
        'March': 'Martius',
        'April': 'Aprilis',
        'May': 'Majus',
        'June': 'Junis',
        'July': 'Julius',
        'August': 'Augustus',
    }
    if month in months_in_latin:
        return months_in_latin[month]
    else:
        return month


def which_sunday(date: datetime) -> int:
    """
    Determine the numeric order of a Sunday within a month.
    """
    sevens = [s for s in range(0, 31, 7)]
    index = int(date.strftime("%d"))
    x = 0
    while index-x not in sevens:
        x += 1
    return sevens.index(index-x)+1


def last_sunday(date: datetime) -> bool:
    if which_sunday(date) < 4:
        return False
    elif which_sunday(date) == 4 and int(date.strftime("%d")) < 25:
        return False
    else:
        return True



from datetime import datetime
from ordotools import LiturgicalCalendar

def test_basic():
    assert LiturgicalCalendar(int(datetime.today().strftime("%Y")), "roman")
    assert LiturgicalCalendar(2024, "roman")
    assert LiturgicalCalendar(2025, "roman")
    assert LiturgicalCalendar(2026, "roman")
    assert LiturgicalCalendar(2027, "roman")
    assert LiturgicalCalendar(2028, "roman")
    assert LiturgicalCalendar(2029, "roman")
    assert LiturgicalCalendar(2030, "roman")
    assert LiturgicalCalendar(2031, "roman")
    assert LiturgicalCalendar(2032, "roman")
    assert LiturgicalCalendar(2033, "roman")
    assert LiturgicalCalendar(2034, "roman")
    assert LiturgicalCalendar(2035, "roman")
    assert LiturgicalCalendar(2036, "roman")
    assert LiturgicalCalendar(2037, "roman")
    assert LiturgicalCalendar(2038, "roman")
    assert LiturgicalCalendar(2039, "roman")
    assert LiturgicalCalendar(2040, "roman")
    assert LiturgicalCalendar(2041, "roman")
    assert LiturgicalCalendar(2042, "roman")
    assert LiturgicalCalendar(2043, "roman")
    assert LiturgicalCalendar(2044, "roman")
    assert LiturgicalCalendar(2045, "roman")
    assert LiturgicalCalendar(2046, "roman")
    assert LiturgicalCalendar(2047, "roman")
    assert LiturgicalCalendar(2048, "roman")
    assert LiturgicalCalendar(2049, "roman")
    assert LiturgicalCalendar(2050, "roman")
    assert LiturgicalCalendar(2051, "roman")
    assert LiturgicalCalendar(2052, "roman")
    assert LiturgicalCalendar(2053, "roman")
    assert LiturgicalCalendar(2054, "roman")
    assert LiturgicalCalendar(2055, "roman")
    assert LiturgicalCalendar(2056, "roman")
    assert LiturgicalCalendar(2057, "roman")
    assert LiturgicalCalendar(2058, "roman")
    assert LiturgicalCalendar(2059, "roman")
    assert LiturgicalCalendar(2060, "roman")
    assert LiturgicalCalendar(2061, "roman")
    assert LiturgicalCalendar(2062, "roman")
    assert LiturgicalCalendar(2063, "roman")
    assert LiturgicalCalendar(2064, "roman")
    assert LiturgicalCalendar(2065, "roman")
    assert LiturgicalCalendar(2066, "roman")
    assert LiturgicalCalendar(2067, "roman")
    assert LiturgicalCalendar(2068, "roman")
    assert LiturgicalCalendar(2069, "roman")
    assert LiturgicalCalendar(2070, "roman")
    assert LiturgicalCalendar(2071, "roman")
    assert LiturgicalCalendar(2072, "roman")
    assert LiturgicalCalendar(2073, "roman")



from ordotools.tools.helpers import LiturgicalYearMarks
from ordotools.tools.helpers import day
from ordotools.tools.helpers import days
from ordotools.tools.helpers import weeks


"""
Commemorations are a bit tricky to implement. Everything
in this file is for determing the "seasonal commemorations"
for each feast, as well as the Fidélium prayer. The non-seasonal
commemorations are then added on top of these and ordering 
the commemorations is handled by the Feast class.
"""


def advent_season(feast):
    feast.com_1["code"] = 99906
    feast.com_2["code"] = 99909
    return feast

def epiphany_octave_to_septuagesima(year: int, feast):
    if feast.date > day(year, 2, 2):
        feast.com_1["code"] = 99911
        feast.com_2["code"] = 99913
    else:
        feast.com_1["code"] = 99907
        feast.com_2["code"] = 99909 # TODO: or for the pope
    return feast

def septuagesima_to_lent(feast):
    # TODO: special secret of BVM if before February 2nd
    feast.com_1["code"] = 99907
    feast.com_2["code"] = 99909 # or for the pope
    return feast

def lent_to_passiontide(feast):
    feast.com_1["code"] = 99911
    feast.com_2["code"] = 99914
    return feast

def passiontide(feast):
    feast.com_1["code"] = 99909 # or for the pope
    return feast

def easter_octave(feast):
    feast.com_1["code"] = 99909 # or for the pope
    return feast

def easter_season(feast):
    feast.com_1["code"] = 99908
    feast.com_2["code"] = 99909 # or for the pope
    return feast

def pentecost_season(feast):
    feast.com_1["code"] = 99911
    feast.com_2["code"] = 99913
    return feast

def seasonal_commemorations(feasts: tuple, year: int) -> tuple:
    mark = LiturgicalYearMarks(year)
    processed_feasts = ()
    month_indicator = ""

    for feast in feasts:
        if feast.rank_n > 15:

            if mark.first_advent < feast.date < mark.christmas:
                feast = advent_season(feast)

            elif mark.christmas+days(19) < feast.date < mark.lent_begins-weeks(3)-days(3):
                feast = epiphany_octave_to_septuagesima(year, feast)

            elif mark.lent_begins-weeks(3)-days(3) < feast.date < mark.lent_begins:
                feast = septuagesima_to_lent(feast)

            elif mark.lent_begins < feast.date <= mark.lent_ends-weeks(2):
                feast = lent_to_passiontide(feast)

            elif mark.lent_ends-weeks(2) < feast.date < mark.lent_ends:
                feast = passiontide(feast)

            elif mark.easter+days(2) < feast.date < mark.easter+days(7):
                feast = easter_octave(feast)

            elif mark.easter+days(7) < feast.date < mark.easter_season_end:
                feast = easter_season(feast)

            elif mark.pentecost_season_start < feast.date < mark.first_advent:
                feast = pentecost_season(feast)

            else:
                pass

            # FIDELIUM
            if feast.rank_n == 23:  # NOTE: can 23 be an impeded Sunday
                if month_indicator == feast.date.strftime("%B"):
                    pass
                else:
                    if month_indicator == "November":
                        pass
                    elif (
                            mark.first_advent < feast.date < mark.christmas or
                            mark.lent_begins < feast.date < mark.lent_ends or 
                            mark.easter_season_start < feast.date < mark.easter_season_end
                        ):
                        pass
                    else:
                        feast.com_2 = {"code": 99912}
                        month_indicator = feast.date.strftime("%B")

                if feast.date.strftime("%w") == 1:
                    if mark.first_advent < feast.date < mark.christmas:
                        pass
                    elif mark.lent_begins < feast.date < mark.lent_ends:
                        pass
                    else:
                        if feast.com_2["code"] == 99912:
                            pass
                        else:
                            feast.com_2 = {"code": 99912}

        else:
            pass

        processed_feasts += (feast,)

    return processed_feasts
